from OpenSSL import crypto


class WsaaCertificate(object):

    def __init__(self, key):
        self.key = key
        self.country_code = None
        self.state_name = None
        self.company_name = None
        self.company_vat = None

    def validate_values(self):
        """ Validamos que esten todos los campos necesarios seteados """
        values = vars(self)
        for value in values:
            if not values.get(value):
                raise AttributeError('Falta configurar alguno de los siguientes campos:\n'
                                     'Codigo de pais, Provincia, Nombre de la empresa o CUIT')

    def generate_certificate_request(self, hash_sign='sha256', ou='odoo'):

        self.validate_values()

        # Utilizamos la libreria de crypto para generar el pedido de certificado
        req = crypto.X509Req()
        req.get_subject().C = self.country_code
        req.get_subject().ST = self.state_name
        req.get_subject().O = self.company_name
        req.get_subject().OU = ou
        req.get_subject().CN = self.company_name
        req.get_subject().serialNumber = 'CUIT {}'.format(self.company_vat)

        # Validamos el formato de la key
        key = crypto.load_privatekey(crypto.FILETYPE_PEM, self.key)
        self.key = crypto.dump_privatekey(crypto.FILETYPE_PEM, key)

        # Firmamos con la key y el hash el certificado
        req.set_pubkey(key)
        req.sign(key, hash_sign)

        return crypto.dump_certificate_request(crypto.FILETYPE_PEM, req)


class WsaaPrivateKey(object):

    def __init__(self, length=2048):
        self.length = length
        self.key = None

    def generate_rsa_key(self):
        pkey = crypto.PKey()
        pkey.generate_key(crypto.TYPE_RSA, self.length)
        self.key = crypto.dump_privatekey(crypto.FILETYPE_PEM, pkey)



import qrcode
import json
import base64


class QrGeneratorElectronicInvoice(object):

    def __init__(self, qr_value):
        self.qr_value = qr_value
        self.qr_img = 0
        self.reverse_color = False

    def generate_qr(self):
        msg_json = str.encode(json.dumps(self.qr_value))
        url = "https://www.afip.gob.ar/fe/qr/?p=%s" % (base64.b64encode(msg_json)).decode('utf-8')
        qr = qrcode.QRCode(
            version=1,
            error_correction=qrcode.constants.ERROR_CORRECT_L,
            box_size=10,
            border=4
        )
        qr.add_data(url)
        qr.make(fit=True)
        if self.reverse_color:
            self.qr_img = qr.make_image(fill_color="white", back_color="black")
        else:
            self.qr_img = qr.make_image(fill_color="black", back_color="white")

    def get_qr_image(self):
        return self.qr_img



# -*- coding: utf-8 -*-
from l10n_ar_api.documents import invoice
from datetime import date, datetime


class FiscalElectronicBondValidator:

    @classmethod
    def validate_invoice(cls, invoice):
        '''
        Valida que los campos de la factura cumplan con los requisitos de la AFIP
        :param invoices: Factura a validar, objeto FiscalElectronicBond
        '''

        cls._validate_document_date(invoice)
        cls._validate_amount(invoice)

    @staticmethod
    def _validate_document_date(invoice):
        assert invoice.document_date, "El documento no tiene fecha"

        invoice_date = datetime.strptime(invoice.document_date, '%Y%m%d').date()

        if abs(date.today() - invoice_date).days > 5:
            raise AttributeError("La fecha del documento debe ser\
                    mayor o menor a 5 dias de la fecha de generacion")

        date_today = date.today().replace(day=1)
        new_invoice_date = invoice_date.replace(day=1)
        if date_today != new_invoice_date:
            raise AttributeError("La fecha del documento no debe exceder el mes de la fecha de\
                                envío del pedido de autorización")

    @staticmethod
    def _validate_amount(invoice):
        if invoice.get_total_items_amount() > invoice.get_total_amount():
            raise AttributeError("La Suma de los items de la factura\
                                 debe ser menor o igual al total de la misma")

class FiscalElectronicBondItem(object):

    def __init__(self, codigo_prod_ncm, description, quantity = None, measurement_unit = None, unit_price = None, bonification = None, iva_id = None):
        self.codigo_prod_ncm = codigo_prod_ncm
        self.description = description
        self.quantity = quantity
        self.measurement_unit = measurement_unit
        self.unit_price = unit_price
        self.bonification = bonification
        self.iva_id = iva_id

    @property
    def total_price(self):
        assert self.quantity, "No existe cantidad para el item"
        assert self.unit_price, "No existe precio para el item"

        return (self.quantity * self.unit_price) - self.bonification


class FiscalElectronicBond(invoice.Invoice):

    def __init__(self, document_code):
        self.zone_id = 0
        self.reception_amount = None
        self.municipal_reception_amount = None
        self.array_items = []

        # Cliente
        self.customer_name = None
        self.customer_address = None
        self.customer_document_type = None
        self.customer_document_number = None

        # Tributos
        self.total_amount = None
        self.pay_off_tax_amount = None
        self.rni_pay_off_tax_amount = None
        self.internal_tax_amount = None
        self.iibb_amount = None

        # Moneda
        self.mon_id = None
        self.mon_cotiz = None

        # Opcionales
        self.array_optionals = []

        # Comprobantes asociados
        self.associated_documents = []

        self._payment_due_date = None

        super(FiscalElectronicBond, self).__init__(document_code)

    # Override
    def get_total_amount(self):
        return self.total_amount

    def get_total_items_amount(self):
        return sum(item.total_price for item in self.array_items)

    def add_item(self, value):
        self.array_items.append(value)

    @property
    def document_date(self):
        return self._document_date

    @document_date.setter
    def document_date(self, value):
        self._document_date = value.strftime('%Y%m%d')

    @property
    def payment_due_date(self):
        return self._payment_due_date

    @payment_due_date.setter
    def payment_due_date(self, value):
        self._payment_due_date = value.strftime('%Y%m%d')


authorization_urls = dict(
    homologation='http://dfe.test.arba.gov.ar/DomicilioElectronico/SeguridadCliente/dfeServicioDescargaPadron.do',
    production='http://dfe.arba.gov.ar/DomicilioElectronico/SeguridadCliente/dfeServicioDescargaPadron.do',
)



# -*- coding: utf-8 -*-
# Segun RG 2485 – Proyecto FE v2.8 - 12/09/2016
import sys
sys.path.append("..")
from zeep import Client
from zeep.plugins import HistoryPlugin
from zeep.wsse.username import UsernameToken
from l10n_ar_api.anmat_webservices import config
from lxml import etree
try:
    from urlparse import urlparse
except ImportError:
    from urllib.parse import urlparse


import traceback
LOCATION = {
    'location_homologation': 'https://localhost:443/trazaenprodmed.WebService',
    'location_production': 'https://localhost:443/trazaprodmed.WebService',
}


class Anmat(object):
    """ Conexion con WS de ANMAT"""

    def __init__(self, login_data, prueba=False, url=None):
        if url:
            self.url = url
        else:
            self.url = config.service_urls.get('ws_anmat_homologation') if prueba else config.service_urls.get('ws_anmat_production')

        self.usuario = login_data.get('usuario')
        self.password = login_data.get('password')
        self.client = None
        self.history = HistoryPlugin()
        self.CodigoTransaccion = self.Excepcion = self.Errores = self.Resultado = self.Traceback = ""
        self.conectar(wsdl=self.url, prueba=prueba)

    def conectar(self, wsdl=None, prueba=False):
        """Conectar cliente soap del web service"""
        try:
            # Analizar transporte y servidor proxy:
            self.wsdl = self.url
            if not wsdl:
                wsdl = self.wsdl
            # Agregar sufijo para descargar descripción del servicio ?WSDL o ?wsdl
            if not wsdl.endswith(self.wsdl[-5:]) and wsdl.startswith("http"):
                wsdl += self.wsdl[-5:]
            self.client = Client(
                self.url,
                wsse=UsernameToken(
                    'testwservice',
                    'testwservicepsw'
                ),
                plugins=[self.history]
            )
            url = urlparse(wsdl)
            location = LOCATION.get('location_homologation').replace("localhost", url.hostname) if prueba else\
                LOCATION.get('location_production').replace("localhost", url.hostname)
            for service in self.client.wsdl.services.values():
                for port in service.ports.values():
                    port.binding_options['address'] = location
            self.client.service._binding_options['address'] = location
            return True
        except:
            info = sys.exc_info()
            ex = traceback.format_exception(info[0], info[1], info[2])
            self.Traceback = ''.join(ex)
            try:
                self.Excepcion = traceback.format_exception_only(info[0], info[1])[0]
            except:
                self.Excepcion = "No disponible"
            raise

    def _analizar_errores(self, res):
        """Comprueba y extrae errores si existen en la respuesta XML"""

        if res.errores:
            self.Errores = "%s: %s" % (res.errores[0]['c_error'], res.errores[0]['d_error'])
        self.Resultado = res.resultado

    def informar_producto(self, datos_de_envio):
        """
        :param datos_de_envio: lista de diccionario con valores de la transaccion
        """
        res = self.client.service.informarProducto(
            transacciones=datos_de_envio, usuario=self.usuario, password=self.password
        )
        self.CodigoTransaccion = res.codigoTransaccion
        self._analizar_errores(res)
        return True

    def cancelar_transaccion(self, codigo_transaccion):
        """
        parameter: codigo_transaccion {
            codigo de transaccion a cancelar
        }
        """
        res = self.client.service.sendCancelacTransacc(
            transaccion=codigo_transaccion, usuario=self.usuario, password=self.password
        )
        self.CodigoTransaccion = res.codigoTransaccion
        self._analizar_errores(res)
        return True

    def cancelar_transaccion_parcial(self, codigo_transaccion, producto_data):
        """
        :param codigo_transaccion codigo de transaccion a cancelar
        :param producto_data diccionario de gtin y serie de producto a cancalar
        """
        res = self.client.service.sendCancelacTransaccParcial(
            transaccion=codigo_transaccion, usuario=self.usuario,
            password=self.password, gtin=producto_data.get('gtin'), serie=producto_data.get('nro_serie')
        )
        self.CodigoTransaccion = res.codigoTransaccion
        self._analizar_errores(res)
        return True

    def get_xml_request(self):
        try:
            return etree.tostring(self.history.last_sent["envelope"], encoding="unicode", pretty_print=True)
        except (IndexError, TypeError):
            pass

    def get_xml_response(self):
        try:
            return etree.tostring(self.history.last_received["envelope"], encoding="unicode", pretty_print=True)
        except (IndexError, TypeError):
            pass



from zeep import Client
from requests import Session
from zeep.transports import Transport

from l10n_ar_api.afip_webservices import config


class Wsaa(object):

    def __init__(self, homologation=True, url=None):
        
        if not url:
            self.url = config.authorization_urls.get('homologation') if homologation\
                else config.authorization_urls.get('production')
        else:
            self.url = url

    def login(self, tra):
        """
        :param tra: TRA que se usara para el logeo
        :return: XML con el login autorizado
        """
        
        try:
            session = Session()
            session.verify = False
            transport = Transport(session=session)
            login_fault = Client(self.url, transport=transport).service.loginCms(tra)
        except Exception as e:
            raise Exception("Error al autenticarse\n{}".format(e))
        
        return login_fault



try:
    from BeautifulSoup import BeautifulSoup
except ImportError:
    from bs4 import BeautifulSoup

import requests
from unidecode import unidecode
from l10n_ar_api.padron.contributor import Contributor


class Banks:

    @staticmethod
    def get_offline_banks_list():
        return [
            ['30500003886', '3', 'BANCO EUROPEO PARA AMERICA LATINA (BEAL) S.A.', 'ELIMINADO', 'Res.42/04(DI SERE'],
            ['30500001735', '7', 'BANCO DE GALICIA Y BUENOS AIRES S.A.U.', 'VIGENTE', 'Si'],
            ['30500010912', '11', 'BANCO DE LA NACION ARGENTINA', 'VIGENTE', 'SDG LTA', 'Si'],
            ['33999242109', '14', 'BANCO DE LA PROVINCIA DE BUENOS AIRES', 'VIGENTE', 'RES.86/10(DISERE)'],
            ['30709447846', '15', 'INDUSTRIAL AND COMMERCIAL BANK OF CHINA (ARGENTINA) S.A.', 'VIGENTE', '052/13(DI SERE)'],
            ['30500005625', '16', 'CITIBANK N.A.', 'VIGENTE', 'SDG LTA'],
            ['30500003193', '17', 'BANCO BBVA ARGENTINA S.A.', 'VIGENTE', 'SDG LTA'],
            ['30999228565', '20', 'BANCO DE LA PROVINCIA DE CORDOBA S.A.', 'VIGENTE', 'R.100/11 (DI SERE)'],
            ['25', 'BANCO SANTANDER S.A.', 'ELIMINADO'],
            ['33500005179', '27', 'BANCO SUPERVIELLE S.A.', 'VIGENTE', '113/05 (DI SERE)', 'Si'],
            ['30999032083', '29', 'BANCO DE LA CIUDAD DE BUENOS AIRES', 'VIGENTE', 'SDG LTA'],
            ['30500011382', '30', 'CENTRAL DE LA REPUBLICA ARGENTINA', 'VIGENTE'],
            ['31', 'CORREO ARGENTINO S.A.', 'ELIMINADO'],
            ['30500006613', '34', 'BANCO PATAGONIA S.A.', 'VIGENTE', '045/05 (DI SERE)'],
            ['30500011072', '44', 'BANCO HIPOTECARIO SA', 'VIGENTE', '079/09 (DI SERE)'],
            ['30500009442', '45', 'BANCO DE SAN JUAN S.A.', 'VIGENTE', 'SDG LTA'],
            ['33999181819', '65', 'BANCO MUNICIPAL DE ROSARIO', 'VIGENTE', 'SDG LTA'],
            ['30500008454', '72', 'BANCO SANTANDER RIO S.A.', 'VIGENTE', 'RES79/07(DI SERE)', 'Si'],
            ['76', 'DE LA NACION ARGENTINA (SUC.IMP.INT.)', 'ELIMINADO'],
            ['78', 'DE LA CIUDAD DE BUENOS AIRES', 'ELIMINADO'],
            ['81', 'BANCO SOCIAL DE CORDOBA', 'ELIMINADO'],
            ['30500012990', '83', 'BANCO DEL CHUBUT S.A.', 'VIGENTE', 'SDG LTA'],
            ['30500098801', '86', 'BANCO DE SANTA CRUZ S.A.', 'VIGENTE', 'SDG LTA'],
            ['30500012516', '93', 'BANCO DE LA PAMPA SOCIEDAD DE ECONOMIA MIXTA', 'VIGENTE', 'RES104/07(DISERE)'],
            ['30500010602', '94', 'BANCO DE CORRIENTES S.A.', 'VIGENTE'],
            ['30500014047', '97', 'BANCO PROVINCIA DEL NEUQUEN S.A.', 'VIGENTE', 'SDG LTA'],
            ['30715899716', '143', 'BRUBANK S.A.U.', 'VIGENTE'],
            ['144', 'BANCO ARGENTINO DE INVERSION S.A.', 'ELIMINADO'],
            ['30522714417', '147', 'BANCO INTERFINANZAS S.A.', 'VIGENTE', 'Res.Nº46/13(DI SERE)'],
            ['33537186009', '150', 'HSBC BANK ARGENTINA S.A.', 'VIGENTE', 'SDG LTA', 'Si'],
            ['162', 'BANCO MAYO COOPERATIVO LIMITADO', 'ELIMINADO'],
            ['30583137943', '165', 'JP MORGAN CHASE BANK NA (SUCURSAL BUENOS AIRES)', 'VIGENTE', 'RES.137/05(DI SERE)'],
            ['30571421352', '191', 'BANCO CREDICOOP COOPERATIVO LIMITADO', 'VIGENTE', 'SDG LTA'],
            ['30576124275', '198', 'BANCO DE VALORES S.A.', 'VIGENTE', 'SDG LTA'],
            ['222', 'CAJA DE VALORES S.A.', 'ELIMINADO'],
            ['231', 'BANCO DE RIO TERCERO COOP.LTDO.', 'ELIMINADO'],
            ['236', 'BANCO DO ESTADO DE SAO PAULO S.A.', 'ELIMINADO'],
            ['30535610440', '247', 'BANCO ROELA S.A.', 'VIGENTE'],
            ['30516420444', '254', 'BANCO MARIVA S.A.', 'VIGENTE', 'SDG LTA'],
            ['256', 'CITICORP BANCO DE INVERSION S.A.', 'ELIMINADO'],
            ['30580189411', '259', 'BANCO ITAU ARGENTINA S.A.', 'VIGENTE', 'Res106/08(DISERE)'],
            ['260', 'BANCO EXTERIOR DE AMERICA S.A.', 'ELIMINADO'],
            ['30500050558', '262', 'BANK OF AMERICA NATIONAL ASSOCIATION', 'VIGENTE'],
            ['30584727566', '266', 'BNP PARIBAS', 'VIGENTE', 'SDG LTA', 'Si'],
            ['30575655781', '268', 'BANCO PROVINCIA DE TIERRA DEL FUEGO', 'VIGENTE', 'SDG LTA'],
            ['30588337843', '269', 'BANCO DE LA REP. ORIENTAL DEL URUGUAY', 'VIGENTE', 'SDG LTA', 'Si'],
            ['274', 'OF CREDIT AND COMMERCE', 'ELIMINADO'],
            ['30534672434', '277', 'BANCO SAENZ  S.A.', 'VIGENTE'],
            ['30534487491', '281', 'BANCO MERIDIAN S.A.', 'VIGENTE', 'R.103/08(DISERE)', 'Si'],
            ['30500010084', '285', 'BANCO MACRO S.A.', 'VIGENTE', '52/07 (DI SERE)'],
            ['287', 'BANCO MEDEFIN UNB S.A.', 'ELIMINADO'],
            ['298', 'BANCO AUSTRAL S.A.', 'ELIMINADO'],
            ['30604731018', '299', 'BANCO COMAFI S.A.', 'VIGENTE', 'SDG LTA'],
            ['30651129083', '300', 'BANCO DE INVERSION Y COMERCIO EXTERIOR S.A.', 'VIGENTE'],
            ['30569151763', '301', 'BANCO PIANO S.A.', 'VIGENTE', 'SDG LTA'],
            ['302', 'BANCO EXTRADER S.A.', 'ELIMINADO'],
            ['30518954241', '303', 'BANCO FINANSUR S.A.', 'ELIMINADO', 'Res.079/17(DI SERE)'],
            ['30657441216', '305', 'BANCO JULIO S.A.', 'VIGENTE', 'SDG LTA'],
            ['306', 'BANCO PRIVADO DE INVERSIONES S.A.', 'ELIMINADO'],
            ['307', 'BANCO MAYORISTA DEL PLATA S.A.', 'ELIMINADO'],
            ['30671859339', '309', 'BANCO RIOJA SOCIEDAD ANÓNIMA UNIPERSONAL', 'VIGENTE', 'RES. 88/16(DI SERE)'],
            ['30677937560', '310', 'BANCO DEL SOL S.A.', 'VIGENTE'],
            ['30670157799', '311', 'NUEVO BANCO DEL CHACO S.A.', 'VIGENTE', '076/09 (DI SERE)'],
            ['30546741636', '312', 'BANCO VOII S.A.', 'VIGENTE'],
            ['30671375900', '315', 'BANCO DE FORMOSA S.A.', 'VIGENTE'],
            ['316', 'BANCO DE MISIONES S.A.', 'ELIMINADO'],
            ['30576614299', '319', 'BANCO CMF S.A.', 'VIGENTE', 'SDG LTA'],
            ['33686664649', '321', 'BANCO DE SANTIAGO DEL ESTERO S.A.', 'VIGENTE'],
            ['30685029959', '322', 'BANCO INDUSTRIAL S.A.', 'VIGENTE', '075/10 (DI SERE)'],
            ['328', 'BANCO MENDOZA S.A.', 'ELIMINADO'],
            ['30692432661', '330', 'NUEVO BANCO DE SANTA FE S.A.', 'VIGENTE', '29/05 (DI SERE)'],
            ['30697306362', '331', 'BANCO CETELEM ARGENTINA S.A.', 'VIGENTE'],
            ['30697265895', '332', 'BANCO DE SERVICIOS FINANCIEROS S.A.', 'VIGENTE'],
            ['30701255557', '336', 'BANCO BRADESCO ARGENTINA SA', 'VIGENTE'],
            ['337', 'BANCO URQUIJO SA', 'ELIMINADO'],
            ['30704960995', '338', 'BANCO DE SERVICIOS Y TRANSACCIONES S.A.', 'VIGENTE'],
            ['30707108343', '339', 'RCI BANQUE S.A.', 'VIGENTE'],
            ['30707227415', '340', 'BACS BANCO DE CREDITO Y SECURITIZACION S.A.', 'VIGENTE'],
            ['30540618263', '341', 'MAS VENTAS S.A.', 'VIGENTE'],
            ['30715654632', '384', 'WILOBANK S.A.', 'VIGENTE'],
            ['33707995519', '386', 'NUEVO BANCO DE ENTRE RIOS S.A.', 'VIGENTE', 'R.74/03 (DI SERE)'],
            ['30517637498', '389', 'BANCO COLUMBIA S.A.', 'VIGENTE'],
            ['30516544542', '405', 'FORD CREDIT CIA. FINAN. S.A.', 'VIGENTE'],
            ['30538006404', '408', 'COMPAÑIA FINANCIERA ARGENTINA S.A.', 'VIGENTE'],
            ['30581385516', '413', 'MONTEMAR CIA. FINAN. S.A.', 'VIGENTE'],
            ['30628284357', '415', 'TRANSATLANTICA COMPAÑIA FINANCIERA S.A.', 'VIGENTE'],
            ['30712331239', '426', 'BANCO BICA S.A.', 'VIGENTE'],
            ['30714195960', '431', 'BANCO COINAG S.A.', 'VIGENTE'],
            ['30542033637', '432', 'BANCO DE COMERCIO S.A.', 'VIGENTE', 'RES 41/17(DI SERE)', 'Si'],
            ['30716226944', '434', 'CREDITO REGIONAL COMPAÑIA FINANCIERA S.A.', 'VIGENTE'],
            ['30716090333', '435', 'BANCO SUCREDITO S.A.', 'VIGENTE'],
            ['30682419578', '437', 'VOLKSWAGEN FINANCIAL SERVICES COMPAÑIA FINANCIERA S.A.', 'VIGENTE'],
            ['30701810852', '438', 'CORDIAL COMPAÑIA FINANCIERA S.A.', 'VIGENTE'],
            ['30692304884', '440', 'FCA Compañía Financiera S.A.', 'VIGENTE'],
            ['30678564822', '441', 'GPAT COMPAÑIA FINANCIERA S.A.', 'VIGENTE'],
            ['30707002294', '442', 'MERCEDES-BENZ COMPAÑIA FINANCIERA ARGENTINA S.A.', 'VIGENTE'],
            ['33707124909', '443', 'ROMBO CIA. FINAN. S.A.', 'VIGENTE'],
            ['30707024859', '444', 'JOHN DEERE CREDIT CIA. FINAN. S.A.', 'VIGENTE'],
            ['30707847367', '445', 'PSA FINANCE ARGENTINA CIA. FINAN. S.A.', 'VIGENTE'],
            ['30709000426', '446', 'TOYOTA COMPAÑIA FINANCIERA DE ARGENTINA S.A.', 'VIGENTE'],
            ['30712592407', '448', 'BANCO DINO S.A.', 'VIGENTE'],
            ['30716395452', '515', 'BANK OF CHINA LIMITED, SUCURSAL BUENOS AIRES', 'VIGENTE'],
            ['33663293309', '992', 'PROVINCANJE SOCIEDAD ANONIMA', 'VIGENTE'],
            ['993', 'AFIP Seti DJ', 'VIGENTE'],
            ['33546663669', '998', 'CONSEJO PROFESIONAL DE CIENCIAS ECONOMICAS DE CABA', 'VIGENTE']
        ]

    @staticmethod
    def get_banks_list():
        """
        Obtiene la lista de bancos desde AFIP utilizando
        la libreria BeautifulSoup para webscraping
        """

        banks = []
        try:
            url = 'http://www.afip.gov.ar/genericos/emisorasGarantias/formularioCompa%C3%B1ias.asp?completo=1&ent=3'
            f = requests.post(url).content
            soup = BeautifulSoup(f)
            table = soup.find('table', attrs={"class": "contenido"})
            for row in table.findAll('tr')[2:]:
                banks.append([td.text.strip() for td in row.findAll('td') if td.text.strip()])
            return banks
        except Exception as e:
            return Banks.get_offline_banks_list()

    @staticmethod
    def get_values(banks_list):
        """
        :param banks_list: Lista de bancos.
        :return: Lista de diccionarios con los valores de cada banco
        """

        values = []
        for bank_list in banks_list:

            bank_values = {}

            if bank_list:

                # Validamos el cuit del banco para comprobar existencia
                if Contributor.is_valid_cuit(bank_list[0]):

                    bank_values['cuit'] = bank_list[0]
                    bank_values['code'] = bank_list[1]
                    bank_values['name'] = unidecode(bank_list[2])
                    values.append(bank_values)

        return values



# -*- coding: utf-8 -*-


class Tribute:

    @staticmethod
    def get_tribute(tribute_type):
        """ Devuelve la instancia del tipo de percepcion deseado """
        if tribute_type == 'gross_income':
            return GrossIncome()
        elif tribute_type == 'profit':
            return Profit()
        else:
            raise NotImplementedError("El tipo de tributo {} No existe".format(tribute_type))


class GrossIncome(object):

    def __init__(self):
        self.percentage = 0.0
        self.minimum_no_aplicable = 0.0
        self.minimum_tax = 0.0

    def calculate_value(self, amount_to_pay):
        """
        Devuelve el valor del calculo de IIBB
        :param amount_to_pay: Importe a pagar sin impuestos
        :return: Base imponible y Valor calculado
        """

        base = amount_to_pay
        value = base * (self.percentage / 100)

        if base < self.minimum_no_aplicable or value < self.minimum_tax:
            value = 0

        return base, value


class Activity:

    def __init__(self, minimum_no_aplicable, minimum_tax, percentage):
        """
        :param minimum_no_aplicable: Minimo no imponible de la actividad
        :param minimum_tax: Importe minimo a retener/percibir
        :param percentage: Porcentaje de la actividad
        """
        self.minimum_no_aplicable = minimum_no_aplicable
        self.minimum_tax = minimum_tax
        self.percentage = percentage


class Profit:

    def __init__(self):
        self.activity = None

    def calculate_value(self, accumulated, amount_to_pay):
        """
        Devuelve el valor a retener/percibir para la actividad
        :param accumulated: Acumulado de pagos necesario para deducir el valor
        :param amount_to_pay: Importe a pagar sin impuestos
        :return: Base imponible y Valor que se debe retener
        """
        if not self.activity:
            raise AttributeError("Agregar actividad en el tributo antes de calcular el valor")

        minimum_no_aplicable = self.activity.minimum_no_aplicable
        # El acumulado se debe restar siempre, excepto en el caso que el acumulado sea mayor al minimo no imponible
        # y no se haya retenido por primera vez (porque no sobrepaso el minimo), entonces en ese caso el valor de la
        # retencion es por la anterior + la actual
        if ((accumulated - minimum_no_aplicable) * self.activity.percentage / 100) >= self.activity.minimum_tax:
            accumulated = minimum_no_aplicable

        base = amount_to_pay + accumulated - minimum_no_aplicable
        value = base * (self.activity.percentage / 100)

        if value < self.activity.minimum_tax:
            value = 0

        return base, value



# -*- coding: utf-8 -*-

from l10n_ar_api.documents import tribute
import pytest


class TestGrossIncome:

    @pytest.fixture(scope="module")
    def gross_income_tribute(self):
        return tribute.Tribute.get_tribute('gross_income')

    def test_calculate_retention_without_amount_to_pay(self, gross_income_tribute):
        values = gross_income_tribute.calculate_value(0)
        assert not values[0]
        assert not values[1]

    def test_calculate_retention_without_percentage_and_amount_to_pay(self, gross_income_tribute):
        values = gross_income_tribute.calculate_value(500.0)
        assert values[0] == 500.0
        assert not values[1]

    def test_calculate_retention_with_percentage_and_amount_to_pay(self, gross_income_tribute):
        gross_income_tribute.percentage = 2.0
        values = gross_income_tribute.calculate_value(500.0)
        assert values[0] == 500.0
        assert values[1] == 10.0

    def test_calculate_retention_with_higher_than_minimum(self, gross_income_tribute):
        gross_income_tribute.percentage = 2.0
        gross_income_tribute.minimum_no_aplicable = 400.0
        values = gross_income_tribute.calculate_value(500.0)
        assert values[0] == 500.0
        assert values[1] == 10.0

    def test_calculate_retention_with_less_than_minimum(self, gross_income_tribute):
        gross_income_tribute.percentage = 2.0
        gross_income_tribute.minimum_no_aplicable = 400
        values = gross_income_tribute.calculate_value(300.0)
        assert values[0] == 300.0
        assert values[1] == 0.0
